# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.db import models
from django.utils import timezone
from django.core.exceptions import ValidationError
from django.contrib.auth.models import User

def validate_mac(mac):
    if mac == '':
        raise ValidationError(u'%s no puede ser vacio' % mac)
    sep = mac.split(':')
    if len(sep) != 6:
        raise ValidationError(u'%s no sigue el formato XX:XX:XX:XX:XX:XX' % mac)
    for i in sep:
        if len(i) != 2:
            raise ValidationError(u'%s no sigue el formato XX:XX:XX:XX:XX:XX' % mac)
    valores_correctos = ":ABCDEFGHYJKLMNOPQRTUVWXYZ0987654321"
    mac = mac.upper()
    for i in mac:
        correcto = False
        for j in valores_correctos:
            if j == i:
                correcto = True
        if correcto == False:
            raise ValidationError(u'Hay algun caracter erroneo en %s ' % mac)

def validate_numRacionesActuales(numRacionesActuales, racionesMax):
    if(numRacionesActuales > racionesMax):
        raise ValidationError(str(numRacionesActuales) + ' no puede ser mayor que el maximo de raciones acptadas por el comedero, ' + str(racionesMax))
    if(numRacionesActuales < 0):
        raise ValidationError(u'%s no puede ser negativo' % numRacionesActuales)

class Comedero(models.Model):
    mac = models.CharField(max_length=17, validators=[validate_mac], primary_key=True)
    nombre =models.CharField(max_length=20, null=False, blank=False, unique=True)
    racionesMax = models.IntegerField()
    numRacionesActuales = models.IntegerField()
    creado = models.DateTimeField(default=timezone.now)
    nuevaProgramacion = models.BooleanField(default=False)
    titular = models.ForeignKey('auth.User', related_name='comederos', on_delete=models.CASCADE, null=False, blank=False)
    tengo_que_servir = models.NullBooleanField()

    def __str__(self):
        return self.nombre

class Programacion(models.Model):
    dia =  models.DateField(null=True, blank = True)
    hora = models.TimeField(null=True, blank=True)
    comedero = models.ForeignKey('Comedero', on_delete=models.CASCADE, null=False, blank=False)
    servida = models.BooleanField(default=False)

    def __str__(self):
        return str(self.dia)+' '+str(self.hora)

class Usuario(models.Model):
    user = models.OneToOneField(User, on_delete=models.CASCADE)
    roles = (('A', 'Administrador'), ('U', 'Usuario'))
    rol = models.CharField(max_length=1, choices=roles, default='U')

    def getRol(self):
        return self.rol
