# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from .models import Usuario, Comedero, Programacion
from .serializers import ComederoSerializer, ProgramacionSerializer, UserSerializer
from rest_framework import generics
from rest_framework import permissions
from .permissions import IsOwnerOrReadOnly
from django.contrib import messages
from django.shortcuts import render, get_object_or_404, redirect
from time import sleep
from django.contrib.auth.models import User
from .forms import DateForm, AddRaciones, EditInfo


def tus_comederos(request):
    if (request.user.is_anonymous):
        return redirect('login')
    else:
        c = Comedero.objects.filter(titular=request.user)
        programacion = {}
        num_programaciones = {}
        for c_i in c:
            programacion[c_i.nombre] = Programacion.objects.filter(comedero=c_i, servida=False).values('hora', 'dia')
            num = len(Programacion.objects.filter(comedero=c_i, servida=False))
            num_programaciones[c_i.nombre] = num

        return render(request, 'hungryFalconryApp/tus_comederos.html',
                      {'comederos': c, 'programacion': programacion, 'num_programaciones': num_programaciones})


def servir(request):
    if (request.user.is_anonymous):
        return redirect('login')
    else:
        mac = request.POST.get('mac')
        comedero = Comedero.objects.get(mac=mac)
        comedero.tengo_que_servir = True
        comedero.save()
        sleep(2)
        return tus_comederos(request)  # render(request, 'hungryFalconryApp/index.html', {"comedero": comedero})


def add_raciones(request, nombre):
    if (request.user.is_anonymous):
        return redirect('login')
    else:
        comedero_model = Comedero.objects.get(nombre=nombre)
        if request.method == "POST":
            form = AddRaciones(request.POST)
            if (form.is_valid()):
                rac = form.save(commit=False)
                if (0 < rac.numRacionesActuales <= comedero_model.racionesMax):
                    comedero_model.numRacionesActuales = rac.numRacionesActuales
                    comedero_model.save()
                else:
                    messages.error(request, 'No se pueden añadir esas raciones', extra_tags='raciones_error')
        else:
            if (comedero_model.numRacionesActuales > 0):
                messages.warning(request, 'Se perderan las anteriores raciones', extra_tags='hay_raciones_previas')
            form = AddRaciones({'numRacionesActuales': comedero_model.racionesMax})
        return render(request, 'hungryFalconryApp/add_raciones.html',
                      {'nombre': nombre, 'form': form, 'numRacionesActuales': comedero_model.numRacionesActuales,
                       'racionesMax': comedero_model.racionesMax})


def editar_info(request, nombre):
    if (request.user.is_anonymous):
        return redirect('login')
    else:
        comedero_model = Comedero.objects.get(nombre=nombre)
        if request.method == "POST":
            form = EditInfo(request.POST)
            if (form.is_valid()):
                nombre = form.save(commit=False)
                comedero_model.nombre = nombre.nombre
                comedero_model.save()
        else:
            form = EditInfo()
        return render(request, 'hungryFalconryApp/editar_info.html',
                      {'nombre': nombre, 'form': form})


def programar_comederos(request, nombre):
    if (request.user.is_anonymous):
        return redirect('login')
    else:
        comedero_model = Comedero.objects.get(nombre=nombre)
        programacion_previa = Programacion.objects.filter(comedero=comedero_model).values('id', 'hora', 'dia', 'servida')
        prog_sin_servir = Programacion.objects.filter(comedero=comedero_model, servida=False)
        if request.method == "POST":
            form = DateForm(request.POST)
            if (form.is_valid()):
                programacion = form.save(commit=False)
                if ((programacion.dia == None) | (programacion.hora == None)):
                    messages.error(request, 'Tienes que añadir una programacion', extra_tags='sin_programacion')
                elif (len(prog_sin_servir) >= comedero_model.numRacionesActuales):
                    messages.error(request, 'No puedes añadir mas programaciones', extra_tags='max_programaciones')
                else:
                    programacion.comedero = comedero_model
                    programacion.save()
                    comedero_model.nuevaProgramacion = True
                    comedero_model.save()
        else:
            form = DateForm()
        return render(request, 'hungryFalconryApp/programar_comederos.html',
                      {'nombre': nombre, 'form': form, 'programacion': programacion_previa})


def borrar_programacion(request, id):
    if (request.user.is_anonymous):
        return redirect('login')
    else:
        prog = get_object_or_404(Programacion, id=id)
        nombre_comedero = prog.comedero.nombre
        if request.method == "POST":
            prog.delete()
        comedero_model = Comedero.objects.get(nombre=nombre_comedero)
        programacion_previa = Programacion.objects.filter(comedero=comedero_model).values('id', 'hora', 'dia', 'servida')
        form = DateForm()
        comedero_model.nuevaProgramacion = True
        comedero_model.save()
        return render(request, 'hungryFalconryApp/programar_comederos.html',
                      {'nombre': nombre_comedero, 'form': form, 'programacion': programacion_previa})


#################

#### APIREST ####

#################

class ComederoList(generics.ListCreateAPIView):
    """
    Devuelve una lista de todos los Comederos.
    """
    permission_classes = (permissions.IsAuthenticatedOrReadOnly,)
    queryset = Comedero.objects.all()
    serializer_class = ComederoSerializer

    def perform_create(self, serializer):
        serializer.save(titular=self.request.user)


class ComederoDetail(generics.RetrieveUpdateDestroyAPIView):
    """
        get:
        Devuelve una instancia de Comederos.

        put:
        Edita una instancia de Comedero.

        delete:
        Elimina una instancia de Comedero.
    """
    permission_classes = (permissions.IsAuthenticatedOrReadOnly, IsOwnerOrReadOnly,)
    queryset = Comedero.objects.all()
    serializer_class = ComederoSerializer


class ProgramacionList(generics.ListCreateAPIView):
    """
        Devuelve una lista de todas las Programaciones de un Comedero.
    """
    queryset = Programacion.objects.all()
    serializer_class = ProgramacionSerializer


class ProgramacionDetail(generics.RetrieveUpdateDestroyAPIView):
    """
        get:
        Devuelve una instancia de Programaciones.

        put:
        Edita una instancia de Programacion.

        delete:
        Elimina una instancia de Programacion.
    """
    queryset = Programacion.objects.all()
    serializer_class = ProgramacionSerializer


class UserList(generics.ListAPIView):
    """
        Devuelve una lista de todos los Usuarios.
    """
    queryset = User.objects.all()
    serializer_class = UserSerializer


class UserDetail(generics.RetrieveAPIView):
    """
        get:
        Devuelve una instancia de Usuario.
        """
    queryset = User.objects.all()
    serializer_class = UserSerializer
